from src.lunar_policy.variables import variable, variable_or_default


class TestVariables():
    def test_variable_not_found(self, monkeypatch):
        monkeypatch.delenv('LUNAR_VAR_TEST', raising=False)
        assert variable('TEST') is None

    def test_variable_found(self, monkeypatch):
        monkeypatch.setenv('LUNAR_VAR_TEST', 'value')
        assert variable('TEST') == 'value'

    def test_variable_case_sensitive(self, monkeypatch):
        monkeypatch.setenv('LUNAR_VAR_TEST', 'value')
        assert variable('test') is None

    def test_variable_or_default_not_found(self, monkeypatch):
        monkeypatch.delenv('LUNAR_VAR_TEST', raising=False)
        assert variable_or_default('TEST', 'default') == 'default'

    def test_variable_or_default_found(self, monkeypatch):
        monkeypatch.setenv('LUNAR_VAR_TEST', 'value')
        assert variable_or_default('TEST', 'default') == 'value'

    def test_ignore_non_lunar_var_prefix(self, monkeypatch):
        monkeypatch.setenv('OTHER_VAR_TEST', 'value')
        assert variable('TEST') is None



import json
import os
import re
from typing import Any, Callable, List
from pathlib import Path as FSPath

from .data import Path, ComponentData
from .result import AssertionResult, Op, CheckStatus, NoDataError


class Check:
    def __init__(
        self,
        name: str,
        description: str | None = None,
        data: ComponentData | None = None
    ):
        self._name = name
        self._description = description
        self._accessed_paths = []
        self._used_vars = []
        self._results = []
        self._submitted = False

        if data is None:
            try:
                path = os.environ["LUNAR_BUNDLE_PATH"]
                data = ComponentData.from_file(FSPath(path))
            except KeyError:
                raise ValueError(
                    "LUNAR_BUNDLE_PATH is not set"
                )
            except ValueError as e:
                raise ValueError(
                    "invalid LUNAR_BUNDLE_PATH"
                ) from e
            except FileNotFoundError:
                raise ValueError(
                    f"LUNAR_BUNDLE_PATH does not exist: {path}"
                )

        if not isinstance(data, ComponentData):
            raise ValueError(
                f"Data must be a SnippetData instance, got {data}"
            )
        self._data = data

    # Context Manager

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        isNoDataError = isinstance(exc_value, NoDataError)

        if exc_value is not None:
            if isNoDataError:
                self._results.append(
                    AssertionResult(
                        op=Op.FAIL,
                        args=[],
                        result=CheckStatus.NO_DATA,
                        failure_message=str(exc_value)
                    )
                )
            else:
                self._results.append(
                    AssertionResult(
                        op=Op.FAIL,
                        args=[],
                        result=CheckStatus.ERROR,
                        failure_message=f"Unexpected error: {exc_value}"
                    )
                )

        self._submit()
        return isNoDataError

    # Properties

    @property
    def name(self) -> str:
        return self._name

    @property
    def status(self) -> CheckStatus:
        if not self._results:
            return CheckStatus.PASS

        has_no_data = False
        has_fail = False
        has_error = False

        for result in self._results:
            if result.result == CheckStatus.ERROR:
                has_error = True
            if result.result == CheckStatus.NO_DATA:
                has_no_data = True
            if result.result == CheckStatus.FAIL:
                has_fail = True

        if has_error:
            return CheckStatus.ERROR
        if has_no_data:
            return CheckStatus.NO_DATA
        elif has_fail:
            return CheckStatus.FAIL

        return CheckStatus.PASS

    @property
    def failure_reasons(self) -> str:
        failure_messages = [
            result.failure_message
            for result in self._results
            if (result.result == CheckStatus.FAIL and
                result.failure_message is not None)
        ]
        return failure_messages

    # JSON Path value getters

    def get(self, path: str) -> Any:
        try:
            jsonPath = Path(path)
        except ValueError as e:
            raise ValueError(f"Invalid JSON path: {path}") from e

        result = self._data.get_merged(jsonPath)
        if result is None:
            raise NoDataError(f"No data found for {path}")

        self._accessed_paths.extend(result.paths)
        return result.value

    def get_or_default(self, path: str, default: Any = None) -> Any:
        try:
            return self.get(path)
        except NoDataError:
            return default

    def get_all(self, path: str) -> List[Any]:
        try:
            jsonPath = Path(path)
        except ValueError as e:
            raise ValueError(f"Invalid JSON path: {path}") from e

        results = self._data.get_all(jsonPath)

        if not results:
            raise NoDataError(f"No data found for {path}")

        self._accessed_paths.extend(p for r in results for p in r.paths)
        return [r.value for r in results]

    def get_all_or_default(self, path: str, default: Any = None) -> List[Any]:
        try:
            return self.get_all(path)
        except NoDataError:
            return default if isinstance(default, list) else [default]

    # Assertions

    def fail(
        self,
        failure_message: str | None = None
    ):
        self._make_assertion(
            Op.FAIL,
            lambda v: False,
            False,
            False,
            failure_message or "Policy Forced Failure"
        )

    def assert_true(
        self,
        value: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.TRUE,
            lambda v: v is True,
            value,
            all_instances,
            failure_message or f"{value} is not true",
        )

    def assert_false(
        self,
        value: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.FALSE,
            lambda v: v is False,
            value,
            all_instances,
            failure_message or f"{value} is not false",
        )

    def assert_equals(
        self,
        value: Any,
        expected: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.EQUALS,
            lambda v: v == expected,
            value,
            all_instances,
            failure_message or f"{value} is not equal to {expected}",
        )

    def assert_contains(
        self,
        value: Any,
        expected: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.CONTAINS,
            lambda v: expected in v,
            value,
            all_instances,
            failure_message or f"{value} does not contain {expected}",
        )

    def assert_greater(
        self,
        value: Any,
        expected: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.GREATER,
            lambda v: v > expected,
            value,
            all_instances,
            failure_message or f"{value} is not greater than {expected}",
        )

    def assert_greater_or_equal(
        self,
        value: Any,
        expected: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.GREATER_OR_EQUAL,
            lambda v: v >= expected,
            value,
            all_instances,
            failure_message or
            f"{value} is not greater than or equal to {expected}",
        )

    def assert_less(
        self,
        value: Any,
        expected: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.LESS,
            lambda v: v < expected,
            value,
            all_instances,
            failure_message or f"{value} is not less than {expected}",
        )

    def assert_less_or_equal(
        self,
        value: Any,
        expected: Any,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.LESS_OR_EQUAL,
            lambda v: v <= expected,
            value,
            all_instances,
            failure_message or
            f"{value} is not less than or equal to {expected}",
        )

    def assert_match(
        self,
        value: Any,
        pattern: str,
        failure_message: str | None = None,
        all_instances: bool = False
    ):
        self._make_assertion(
            Op.MATCH,
            lambda v: re.match(pattern, v) is not None,
            value,
            all_instances,
            failure_message or f"{value} does not match {pattern}",
        )

    # Private methods

    def _submit(self):
        if not self._submitted:
            output = {
                "name": self._name,
                "assertions": [
                    result.toJson()
                    for result in self._results
                ]
            }

            output["paths"] = list(dict.fromkeys(
                p if p.startswith('.') else f'.{p}'
                for p in self._accessed_paths
            ))

            if self._description is not None:
                output["description"] = self._description

            print(json.dumps(output), end="")

            self._submitted = True

    def _get_query_result(
        self,
        value: Any,
        all_instances: bool
    ) -> tuple[Any, list[str]]:
        if not isinstance(value, Path):
            return value, []

        if self._data is None:
            return None, []

        if all_instances:
            results = self._data.get_all(value)
            if not results:
                return None, []
            values = [r.value for r in results]
            paths = [p for r in results for p in r.paths]
            return values, paths
        else:
            result = self._data.get_merged(value)
            if result is None:
                return None, []
            return result.value, result.paths

    def _make_assertion(
        self,
        op: Op,
        check_fn: Callable[[Any], bool],
        value: Any,
        all_instances: bool,
        failure_message: str,
    ) -> AssertionResult:
        actual_value, paths = self._get_query_result(value, all_instances)
        self._accessed_paths.extend(paths)

        if actual_value is None:
            if isinstance(value, Path):
                raise NoDataError(f"No data found for {value.full_path}")
            else:
                raise NoDataError("Value was None")

        if all_instances:
            if not isinstance(actual_value, list):
                actual_value = [actual_value]
            ok = all(check_fn(v) for v in actual_value)
        else:
            ok = check_fn(actual_value)

        try:
            # We don't care about the actual value,
            # we just want to make sure its serializable at submission time
            serialized_value = str(actual_value)
        except Exception as e:
            type_name = type(actual_value).__name__
            serialized_value = f"<typename {type_name}: {e}>"

        self._results.append(
            AssertionResult(
                op=op,
                args=[serialized_value],
                result=CheckStatus.PASS if ok else CheckStatus.FAIL,
                failure_message=failure_message if not ok else None,
            )
        )



import json
import pytest

from src.lunar_policy import Check, Path, ComponentData


class TestCheckPaths:
    def test_path_in_check(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": True,
        }))

        with Check("test", data=data) as c:
            c.assert_true(Path(".value"))

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result["paths"] == [".value"]

    def test_multiple_paths_in_check(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "true_value": True,
            "false_value": False
        }))

        with Check("test", data=data) as c:
            v = c.get(".false_value")
            c.assert_false(v)
            c.assert_true(Path(".true_value"))

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result["paths"] == [".false_value", ".true_value"]

    def test_money_path(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "true_value": True,
            "false_value": False
        }))

        with Check("test", data=data) as c:
            v = c.get(".false_value")
            c.assert_false(v)
            c.assert_true(Path(".true_value"))

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result["paths"] == [".false_value", ".true_value"]

    def test_tricky_path(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "tricky_value": ".true_value",
            ".true_value": True
        }))

        with Check("test", data=data) as c:
            v = c.get("$.tricky_value")
            c.assert_equals(v, ".true_value")

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result["paths"] == [".tricky_value"]

    def test_paths_not_in_check(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result["paths"] == []

    def test_get_invalid_path(self):
        data = ComponentData.from_component_json("{}")

        with pytest.raises(ValueError):
            with Check("test", data=data) as c:
                c.get("@#&*(!%)")



import pytest
import json
import tempfile
from pathlib import Path as FSPath
from src.lunar_policy.data import ComponentData, Path


def assert_result(result, expected_value, expected_paths):
    assert result is not None
    assert result.value == expected_value
    assert result.paths == expected_paths


def assert_all_results(result, expected):
    assert result is not None
    assert len(result) == len(expected)
    for idx, (expected_value, expected_paths) in enumerate(expected):
        assert result[idx].value == expected_value
        assert result[idx].paths == expected_paths


class TestDataLoading():
    def test_from_component_json(self):
        component_json = {
            'string': 'hello world'
        }
        data = ComponentData.from_component_json(json.dumps(component_json))
        assert_result(
            data.get_merged(Path('.string')),
            'hello world',
            ['string']
        )
        assert_all_results(
            data.get_all(Path('.string')),
            [('hello world', ['string'])]
        )

    def test_from_json(self):
        test_data = {
            'metadata_instances': [
                {
                    'payload': {
                        'string': 'hello world'
                    }
                }
            ],
            'merged_blob': {
                'string': 'hello world'
            }
        }
        data = ComponentData.from_json(json.dumps(test_data))
        assert_result(
            data.get_merged(Path('.string')),
            'hello world',
            ['string']
        )
        assert_all_results(
            data.get_all(Path('.string')),
            [('hello world', ['string'])]
        )

    def test_from_file(self):
        test_data = {
            'metadata_instances': [
                {
                    'payload': {
                        'string': 'hello world'
                    }
                }
            ],
            'merged_blob': {
                'string': 'hello world'
            }
        }

        with tempfile.NamedTemporaryFile(
            'w+', delete=False, suffix='.json'
        ) as tmpfile:
            tmpfile.write(json.dumps(test_data))
            tmpfile.flush()
            tmp_path = FSPath(tmpfile.name)

        data = ComponentData.from_file(FSPath(tmp_path))
        assert_result(
            data.get_merged(Path('.string')),
            'hello world',
            ['string']
        )
        assert_all_results(
            data.get_all(Path('.string')),
            [('hello world', ['string'])]
        )


class TestGetMerged():
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'metadata_instances': [],
            'merged_blob': {
                'string': 'hello world',
                'array': ['hello', 'world'],
                'object1': {'hello': 'world'},
                'object2': {'hello': 'moon'}
            }
        }
        return ComponentData.from_json(json.dumps(test_data))

    def test_get_single_value(self, snippet_data):
        result = snippet_data.get_merged(Path('.string'))
        assert_result(result, 'hello world', ['string'])

    def test_get_missing(self, snippet_data):
        result = snippet_data.get_merged(Path('.missing'))
        assert result is None

    def test_get_array(self, snippet_data):
        result = snippet_data.get_merged(Path('.array'))
        assert_result(result, ['hello', 'world'], ['array'])

    def test_get_array_index(self, snippet_data):
        result = snippet_data.get_merged(Path('.array[0]'))
        assert_result(result, 'hello', ['array.[0]'])

    def test_get_nested_object(self, snippet_data):
        result = snippet_data.get_merged(Path('.object1.hello'))
        assert_result(result, 'world', ['object1.hello'])

    def test_get_nested_object_missing(self, snippet_data):
        result = snippet_data.get_merged(
            Path('.object1.missing')
        )
        assert result is None

    def test_multi_match_array_index(self, snippet_data):
        result = snippet_data.get_merged(Path('.array[*]'))
        assert_result(
            result,
            ['hello', 'world'],
            ['array.[0]', 'array.[1]']
        )

    def test_multi_match_object_key(self, snippet_data):
        result = snippet_data.get_merged(Path('..hello'))
        assert_result(
            result,
            ['world', 'moon'],
            ['object1.hello', 'object2.hello']
        )


class TestGetAll():
    @pytest.fixture
    def snippet_data(self):
        test_data = {
            'merged_blob': {},
            'metadata_instances': [
                {
                    'payload': {
                        'single': 'hello world',
                        'double': 'hello1',
                        'single_array': ['hello', 'world'],
                        'double_array': ['goodbye', 'moon'],
                        'single_object': {'hello': 'world'},
                        'double_object': {'hello': 'moon'}
                    }
                },
                {
                    'payload': {
                        'double': 'hello2',
                        'double_array': ['mars', 'i', 'guess'],
                        'double_object': {'hello': 'venus'}
                    }
                }
            ]
        }
        return ComponentData.from_json(json.dumps(test_data))

    def test_get_single_key_in_one_delta(self, snippet_data):
        result = snippet_data.get_all(Path('.single'))
        assert_all_results(result, [
            ('hello world', ['single'])
        ])

    def test_get_single_key_in_two_deltas(self, snippet_data):
        result = snippet_data.get_all(Path('.double'))
        assert_all_results(result, [
            ('hello2', ['double']),
            ('hello1', ['double'])
        ])

    def test_get_missing(self, snippet_data):
        result = snippet_data.get_all(Path('.missing'))
        assert_all_results(result, [])

    def test_get_single_array(self, snippet_data):
        result = snippet_data.get_all(Path('.single_array'))
        assert_all_results(result, [
            (['hello', 'world'], ['single_array'])
        ])

    def test_get_single_array_index(self, snippet_data):
        result = snippet_data.get_all(Path('.single_array[0]'))
        assert_all_results(result, [
            ('hello', ['single_array.[0]'])
        ])

    def test_get_double_array(self, snippet_data):
        result = snippet_data.get_all(Path('.double_array'))
        assert_all_results(result, [
            (['mars', 'i', 'guess'], ['double_array']),
            (['goodbye', 'moon'], ['double_array'])
        ])

    def test_get_double_array_index(self, snippet_data):
        result = snippet_data.get_all(Path('.double_array[0]'))
        assert_all_results(result, [
            ('mars', ['double_array.[0]']),
            ('goodbye', ['double_array.[0]'])
        ])

    def test_get_double_array_index_some_out_of_bounds(self, snippet_data):
        result = snippet_data.get_all(Path('.double_array[2]'))
        assert_all_results(result, [
            ('guess', ['double_array.[2]'])
        ])

    def test_get_single_object(self, snippet_data):
        result = snippet_data.get_all(Path('.single_object'))
        assert_all_results(result, [
            ({'hello': 'world'}, ['single_object'])
        ])

    def test_get_double_object(self, snippet_data):
        result = snippet_data.get_all(Path('.double_object'))
        assert_all_results(result, [
            ({'hello': 'venus'}, ['double_object']),
            ({'hello': 'moon'}, ['double_object'])
        ])

    def test_get_single_object_key(self, snippet_data):
        result = snippet_data.get_all(
            Path('.single_object.hello')
        )
        assert_all_results(result, [
            ('world', ['single_object.hello'])
        ])

    def test_get_double_object_key(self, snippet_data):
        result = snippet_data.get_all(
            Path('.double_object.hello')
        )
        assert_all_results(result, [
            ('venus', ['double_object.hello']),
            ('moon', ['double_object.hello'])
        ])

    def test_get_single_object_key_missing(self, snippet_data):
        result = snippet_data.get_all(
            Path('.single_object.missing')
        )
        assert_all_results(result, [])

    def test_get_double_object_key_missing(self, snippet_data):
        result = snippet_data.get_all(
            Path('.double_object.missing')
        )
        assert_all_results(result, [])

    def test_multi_match_single_array_index(self, snippet_data):
        result = snippet_data.get_all(Path('.single_array[*]'))
        assert_all_results(result, [
            (['hello', 'world'], ['single_array.[0]', 'single_array.[1]'])
        ])

    def test_multi_match_double_array_index(self, snippet_data):
        result = snippet_data.get_all(Path('.double_array[*]'))
        assert_all_results(result, [
            (
                ['mars', 'i', 'guess'],
                ['double_array.[0]', 'double_array.[1]', 'double_array.[2]']
            ),
            (
                ['goodbye', 'moon'],
                ['double_array.[0]', 'double_array.[1]']
            )
        ])

    def test_multi_match_object_key(self, snippet_data):
        result = snippet_data.get_all(Path('..hello'))
        assert_all_results(result, [
            ('venus', ['double_object.hello']),
            (['world', 'moon'], ['single_object.hello', 'double_object.hello'])
        ])


class TestJsonPathExpression():
    def test_invalid_json_path(self):
        with pytest.raises(ValueError):
            Path('.[invalid')

    def test_invalid_snippet_data(self):
        with pytest.raises(ValueError):
            ComponentData({})

    def test_invalid_snippet_data_missing_merged_blob(self):
        with pytest.raises(ValueError):
            ComponentData({
                'metadata_instances': []
            })

    def test_invalid_snippet_data_missing_metadata_instances(self):
        with pytest.raises(ValueError):
            ComponentData({
                'merged_blob': {}
            })



import json

from src.lunar_policy import Check, Path, ComponentData
from src.lunar_policy.result import NoDataError


class TestCheckNoData:
    def test_can_report_no_data(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(Path(".not.a.path"))

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert len(results) == 1
        assert results[0]["result"] == "no_data"
        assert ".not.a.path" in results[0]["failure_message"]
        assert results[0]["op"] == "fail"

    def test_can_report_no_data_with_get(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.get(".not.a.path")

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert len(results) == 1
        assert results[0]["result"] == "no_data"
        assert ".not.a.path" in results[0]["failure_message"]
        assert results[0]["op"] == "fail"

    def test_can_report_no_data_with_get_all(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.get_all(".not.a.path")

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert len(results) == 1
        assert results[0]["result"] == "no_data"
        assert ".not.a.path" in results[0]["failure_message"]
        assert results[0]["op"] == "fail"

    def test_assert_none_is_no_data(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(None)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert len(results) == 1
        assert results[0]["result"] == "no_data"
        assert "None" in results[0]["failure_message"]
        assert results[0]["op"] == "fail"

    def test_exit_check_early_on_no_data(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": True,
        }))

        with Check("test", data=data) as c:
            c.assert_true(Path(".value"))
            c.assert_false(Path(".not.a.path"))
            c.assert_equals("should not run", "should not run")

        captured = capsys.readouterr()
        result = json.loads(captured.out)
        assert len(result["assertions"]) == 2
        assert all(a["op"] != "equals" for a in result["assertions"])

    def test_exit_check_early_on_no_data_get(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.get(".not.a.path")
            c.assert_equals("should not run", "should not run")

        captured = capsys.readouterr()
        result = json.loads(captured.out)
        assert len(result["assertions"]) == 1

    def test_suppress_on_no_data(self, capsys):
        data = ComponentData.from_component_json("{}")
        with Check("test", data=data) as c:
            try:
                c.assert_true(Path(".not.a.path"))
            except NoDataError:
                pass

            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)
        assert len(result["assertions"]) == 1

    def test_suppress_on_no_data_get(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            try:
                c.get(".not.a.path")
            except NoDataError:
                pass

            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)
        assert len(result["assertions"]) == 1



from .data import Path, ComponentData
from .check import Check
from .result import Op, CheckStatus, AssertionResult
from .variables import variable, variable_or_default

__version__ = "0.1.4"

__all__ = [
    # Loading Data
    "Path",
    "ComponentData",

    # Making Assertions
    "Check",

    # Result Types
    "Op",
    "CheckStatus",
    "AssertionResult",

    # Snippet Variables
    "variable",
    "variable_or_default"
]



import pytest

from src.lunar_policy import Check, CheckStatus, ComponentData, Path


class TestCheckName:
    def test_name(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(True)

        assert c.name == "test"


class TestCheckFailureReasons:
    def test_failure_reasons_single(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.fail("this failed")

        assert c.failure_reasons == ["this failed"]

    def test_failure_reasons_multiple(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.fail("this failed")
            c.fail("this failed too")

        assert c.failure_reasons == ["this failed", "this failed too"]


class TestCheckStatus:
    def test_status_pass(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(True)

        assert c.status == CheckStatus.PASS

    def test_status_fail(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.fail("this failed")

        assert c.status == CheckStatus.FAIL

    def test_status_no_data(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(Path(".not.a.path"))

        assert c.status == CheckStatus.NO_DATA

    def test_status_error(self):
        data = ComponentData.from_component_json("{}")

        with pytest.raises(Exception):
            with Check("test", data=data) as c:
                raise Exception("this is an error")

        assert c.status == CheckStatus.ERROR

    def test_status_no_assertions(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            pass

        assert c.status == CheckStatus.PASS

    def test_fail_before_pass(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(True)
            c.fail("this failed")

        assert c.status == CheckStatus.FAIL

    def test_no_data_before_pass(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(True)
            c.assert_true(Path(".not.a.path"))

        assert c.status == CheckStatus.NO_DATA

    def test_no_data_before_fail(self):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.fail("this failed")
            c.assert_true(Path(".not.a.path"))

        assert c.status == CheckStatus.NO_DATA



from enum import Enum
from dataclasses import dataclass
from typing import Any, Optional
from dataclasses import asdict


class NoDataError(Exception):
    pass


class CheckStatus(Enum):
    PASS = "pass"
    FAIL = "fail"
    NO_DATA = "no_data"
    ERROR = "error"


class Op(Enum):
    CONTAINS = "contains"
    EQUALS = "equals"
    TRUE = "true"
    FALSE = "false"
    GREATER = "greater"
    GREATER_OR_EQUAL = "greater_or_equal"
    LESS = "less"
    LESS_OR_EQUAL = "less_or_equal"
    MATCH = "match"
    FAIL = "fail"
    ERROR = "error"


@dataclass
class AssertionResult:
    op: Op
    args: list[Any]
    result: CheckStatus
    failure_message: Optional[str] = None

    def toJson(self):
        result = asdict(self)

        result['op'] = self.op.value
        result['result'] = self.result.value

        if result['failure_message'] is None:
            del result['failure_message']

        return result



import json
import pytest
import tempfile

from src.lunar_policy import Check, Path


class TestCheckBundle:
    @pytest.fixture(autouse=True)
    def setup_lunar_bundle(self, monkeypatch):
        test_data = {
            'merged_blob': {
                'merged_true': True,
                'merged_false': False,
                'value_in_both_roots': "hello world",
                'tricky_value': ".merged_true"
            },
            'metadata_instances': [
                {
                    'payload': {
                        'value_in_both_roots': "hello moon",
                        'value_in_one_delta': "one",
                        'value_in_two_deltas': "two_a"
                    }
                },
                {
                    'payload': {
                        'value_in_both_roots': "hello moon",
                        'value_in_two_deltas': "two_b"
                    }
                }
            ]
        }

        with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
            json.dump(test_data, f)
            temp_path = f.name

        monkeypatch.setenv('LUNAR_BUNDLE_PATH', temp_path)

        yield

    def test_value_in_both_roots(self, capsys):
        with Check("test") as c:
            c.assert_equals(
                Path(".value_in_both_roots"),
                "hello world",
                all_instances=False
            )
            c.assert_equals(
                Path(".value_in_both_roots"),
                "hello moon",
                all_instances=True
            )

        captured = capsys.readouterr()
        results = json.loads(captured.out)
        assert all(
            result["result"] == "pass"
            for result in results["assertions"]
        )
        assert results["paths"] == [".value_in_both_roots"]

    def test_value_in_single_delta(self, capsys):
        with Check("test") as c:
            c.assert_equals(
                Path(".value_in_one_delta"),
                "one",
                all_instances=True
            )

        captured = capsys.readouterr()
        results = json.loads(captured.out)
        assert all(
            result["result"] == "pass"
            for result in results["assertions"]
        )
        assert results["paths"] == [".value_in_one_delta"]

    def test_value_in_two_deltas(self, capsys):
        with Check("test") as c:
            c.assert_contains(
                Path(".value_in_two_deltas"),
                "two",
                all_instances=True
            )

        captured = capsys.readouterr()
        results = json.loads(captured.out)
        assert all(
            result["result"] == "pass"
            for result in results["assertions"]
        )
        assert results["paths"] == [".value_in_two_deltas"]



import json
import pytest

from src.lunar_policy import Check, ComponentData


class TestCheckBasic:
    def test_invalid_check_initialization(self):
        with pytest.raises(ValueError):
            Check("test", data="not a SnippetData object")

    def test_data_error_if_no_env(self,):
        with pytest.raises(ValueError):
            Check("test", data=None)

    def test_data_error_if_invalid_path(self, monkeypatch):
        monkeypatch.setenv('LUNAR_BUNDLE_PATH', "/invalid/path")

        with pytest.raises(ValueError):
            Check("test")

    def test_data_is_set(self, capsys):
        dataJson = {"hi": "there"}
        data = ComponentData.from_component_json(json.dumps(dataJson))

        with Check("test", data=data) as c:
            c.get(".hi")

    def test_description_check(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", "description", data=data) as c:
            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert result["description"] == "description"

    def test_description_not_in_check(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)

        assert "description" not in result



from jsonpath_ng import parse
from pathlib import Path as FSPath
import json
from typing import Any, List
from dataclasses import dataclass


class Path:
    def __init__(self, path: str):
        if not path.startswith("$"):
            path = f"${path}"

        self.full_path = path
        try:
            self._parsed = parse(path)
        except Exception as e:
            raise ValueError(f"Invalid JSON path: {path}") from e

    def find(self, payload: Any):
        return self._parsed.find(payload)


@dataclass
class QueryResult:
    value: Any
    paths: List[str]


class ComponentData:
    def __init__(self, data: dict):
        if not data:
            raise ValueError("data dictionary cannot be empty")
        if 'merged_blob' not in data:
            raise ValueError("data must contain 'merged_blob' key")
        if 'metadata_instances' not in data:
            raise ValueError("data must contain 'metadata_instances' key")
        self._data = data

    @classmethod
    def from_file(cls, path: FSPath) -> 'ComponentData':
        if not path.is_absolute():
            raise ValueError("path must be an absolute path")

        with path.open() as f:
            data = json.load(f)

        return cls(data)

    @classmethod
    def from_json(cls, json_str: str) -> 'ComponentData':
        data = json.loads(json_str)
        return cls(data)

    @classmethod
    def from_component_json(cls, json_str: str) -> 'ComponentData':
        componentJSON = json.loads(json_str)
        return cls({
            'merged_blob': componentJSON,
            'metadata_instances': [
                {
                    'payload': componentJSON
                }
            ]
        })

    def get_merged(self, query: Path) -> QueryResult | None:
        if not isinstance(query, Path):
            raise ValueError("query must be a JsonPathExpression")

        payload = self._data.get('merged_blob', {})
        matches = list(query.find(payload))

        if len(matches) == 0:
            return None

        return QueryResult(
            value=(matches[0].value
                   if len(matches) == 1
                   else [match.value for match in matches]),
            paths=[str(match.full_path) for match in matches]
        )

    def get_all(self, query: Path) -> List[QueryResult]:
        if not isinstance(query, Path):
            raise ValueError("query must be a JsonPathExpression")

        deltas = self._data.get('metadata_instances', [])
        results = []

        for instance in reversed(deltas):
            payload = instance.get('payload', {})
            matches = list(query.find(payload))

            if len(matches) == 0:
                continue

            results.append(QueryResult(
                value=(matches[0].value
                       if len(matches) == 1
                       else [match.value for match in matches]),
                paths=[str(match.full_path) for match in matches]
            ))

        return results



import json
import pytest

from src.lunar_policy import Check, ComponentData


class TestCheckGet:
    def test_get_valid_path(self):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"
        }))

        with Check("test", data=data) as c:
            c.get(".hi")

    def test_get_invalid_path(self):
        data = ComponentData.from_component_json("{}")

        with pytest.raises(ValueError):
            with Check("test", data=data) as c:
                c.get("@#$@#$@#$")

    def test_get_or_default(self):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"
        }))

        exists, default = "", ""
        with Check("test", data=data) as c:
            exists = c.get_or_default(".hi", "default")
            default = c.get_or_default(".not.a.path", "default")

        assert exists == "there"
        assert default == "default"

    def test_get_all_valid_path_single(self):
        data = ComponentData.from_json(json.dumps({
            "merged_blob": {},
            "metadata_instances": [
                {
                    "payload": {
                        "hi": "there"
                    }
                }
            ]
        }))

        all = []
        with Check("test", data=data) as c:
            all = c.get_all(".hi")

        assert all == ["there"]

    def test_get_all_valid_path_multiple(self):
        data = ComponentData.from_json(json.dumps({
            "merged_blob": {},
            "metadata_instances": [
                {
                    "payload": {
                        "hi": "there"
                    }
                },
                {
                    "payload": {
                        "hi": "there"
                    }
                }
            ]
        }))

        all = []
        with Check("test", data=data) as c:
            all = c.get_all(".hi")

        assert all == ["there", "there"]

    def test_get_all_invalid_path(self):
        data = ComponentData.from_component_json("{}")

        with pytest.raises(ValueError):
            with Check("test", data=data) as c:
                c.get_all("@#$@#$@#$")

    def test_get_all_or_default(self):
        data = ComponentData.from_json(json.dumps({
            "merged_blob": {},
            "metadata_instances": [
                {
                    "payload": {
                        "hi": "there"
                    }
                }
            ]
        }))

        exists, default = "", ""
        with Check("test", data=data) as c:
            exists = c.get_all_or_default(".hi", "default")
            default = c.get_all_or_default(".not.a.path", "default")

        assert exists == ["there"]
        assert default == ["default"]



import os

prefix = "LUNAR_VAR_"


def variable(var_key):
    for env_key, value in os.environ.items():
        if env_key.startswith(prefix):
            trimmed_key = env_key[len(prefix):]  # Remove prefix
            if trimmed_key == var_key:
                return value

    return None


def variable_or_default(var_key, default):
    value = variable(var_key)
    if value is None:
        return default
    return value



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hub.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\thub.proto\x12\x06hubapi\x1a\x1fgoogle/protobuf/timestamp.proto\"I\n\x18RunCodeCollectorsRequest\x12-\n\x13\x63ommit_max_age_secs\x18\x01 \x01(\x03R\x10\x63ommitMaxAgeSecs\"M\n\x11\x43odeCollectionRun\x12\x19\n\x08repo_uri\x18\x01 \x01(\tR\x07repoUri\x12\x1d\n\ncommit_sha\x18\x02 \x01(\tR\tcommitSha\"h\n\x19RunCodeCollectorsResponse\x12K\n\x14\x63ode_collection_runs\x18\x01 \x03(\x0b\x32\x19.hubapi.CodeCollectionRunR\x12\x63odeCollectionRuns\"F\n\x0e\x43ollectRequest\x12\x34\n\x0b\x63ollections\x18\x01 \x03(\x0b\x32\x12.hubapi.CollectionR\x0b\x63ollections\"\x11\n\x0f\x43ollectResponse\"2\n\x13PullManifestRequest\x12\x1b\n\tmain_repo\x18\x01 \x01(\tR\x08mainRepo\".\n\x14PullManifestResponse\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\"\x9e\x03\n\nCollection\x12#\n\rcollection_id\x18\x01 \x01(\tR\x0c\x63ollectionId\x12\x12\n\x04\x62lob\x18\x02 \x01(\tR\x04\x62lob\x12;\n\x0bobserved_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\nobservedAt\x12!\n\x0c\x63ollector_id\x18\x04 \x01(\tR\x0b\x63ollectorId\x12\x30\n\x06source\x18\x05 \x01(\x0e\x32\x18.hubapi.CollectionSourceR\x06source\x12\x42\n\ndimensions\x18\x06 \x03(\x0b\x32\".hubapi.Collection.DimensionsEntryR\ndimensions\x12!\n\x0c\x63omponent_id\x18\x07 \x01(\tR\x0b\x63omponentId\x12\x1f\n\x0b\x64uration_ms\x18\x08 \x01(\x03R\ndurationMs\x1a=\n\x0f\x44imensionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x17\n\x15\x43urrentVersionRequest\"M\n\x16\x43urrentVersionResponse\x12\x18\n\x07version\x18\x01 \x01(\tR\x07version\x12\x19\n\x08repo_uri\x18\x02 \x01(\tR\x07repoUri\"\x88\x01\n\x1b\x43heckComponentStatusRequest\x12!\n\x0c\x63omponent_id\x18\x01 \x01(\tR\x0b\x63omponentId\x12\x10\n\x03sha\x18\x02 \x01(\tR\x03sha\x12\x34\n\x04type\x18\x03 \x01(\x0e\x32 .hubapi.CheckComponentStatusTypeR\x04type\".\n\x1c\x43heckComponentStatusResponse\x12\x0e\n\x02ok\x18\x01 \x01(\x08R\x02ok\"\xb0\x01\n\x0f\x41ssertionResult\x12\x1a\n\x02op\x18\x01 \x01(\x0e\x32\n.hubapi.OpR\x02op\x12\x12\n\x04\x61rgs\x18\x02 \x03(\tR\x04\x61rgs\x12+\n\x06result\x18\x03 \x01(\x0e\x32\x13.hubapi.CheckStatusR\x06result\x12,\n\x0f\x66\x61ilure_message\x18\x04 \x01(\tH\x00R\x0e\x66\x61ilureMessage\x88\x01\x01\x42\x12\n\x10_failure_message\"l\n\x14\x43ollectPolicyRequest\x12\x1b\n\tpolicy_id\x18\x01 \x01(\tR\x08policyId\x12\x37\n\nassertions\x18\x02 \x03(\x0b\x32\x17.hubapi.AssertionResultR\nassertions\"1\n\x15\x43ollectPolicyResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success*g\n\x10\x43ollectionSource\x12\x1d\n\x19\x43OLLECTION_SOURCE_UNKNOWN\x10\x00\x12\x18\n\x14\x43OLLECTION_SOURCE_CI\x10\x01\x12\x1a\n\x16\x43OLLECTION_SOURCE_CODE\x10\x02*\x90\x01\n\x18\x43heckComponentStatusType\x12\'\n#CHECK_COMPONENT_STATUS_TYPE_UNKNOWN\x10\x00\x12\'\n#CHECK_COMPONENT_STATUS_TYPE_RELEASE\x10\x01\x12\"\n\x1e\x43HECK_COMPONENT_STATUS_TYPE_PR\x10\x02*\x87\x01\n\x0b\x43heckStatus\x12\x18\n\x14\x43HECK_STATUS_UNKNOWN\x10\x00\x12\x15\n\x11\x43HECK_STATUS_PASS\x10\x01\x12\x15\n\x11\x43HECK_STATUS_FAIL\x10\x02\x12\x18\n\x14\x43HECK_STATUS_NO_DATA\x10\x03\x12\x16\n\x12\x43HECK_STATUS_ERROR\x10\x04*\xc4\x01\n\x02Op\x12\x0e\n\nOP_UNKNOWN\x10\x00\x12\x0f\n\x0bOP_CONTAINS\x10\x01\x12\r\n\tOP_EQUALS\x10\x02\x12\x0b\n\x07OP_TRUE\x10\x03\x12\x0c\n\x08OP_FALSE\x10\x04\x12\x0e\n\nOP_GREATER\x10\x05\x12\x17\n\x13OP_GREATER_OR_EQUAL\x10\x06\x12\x0b\n\x07OP_LESS\x10\x07\x12\x14\n\x10OP_LESS_OR_EQUAL\x10\x08\x12\x0c\n\x08OP_MATCH\x10\t\x12\x0b\n\x07OP_FAIL\x10\n\x12\x0c\n\x08OP_ERROR\x10\x0b\x32\xf6\x03\n\x03Hub\x12<\n\x07\x43ollect\x12\x16.hubapi.CollectRequest\x1a\x17.hubapi.CollectResponse\"\x00\x12M\n\x0cPullManifest\x12\x1b.hubapi.PullManifestRequest\x1a\x1c.hubapi.PullManifestResponse\"\x00\x30\x01\x12Z\n\x11RunCodeCollectors\x12 .hubapi.RunCodeCollectorsRequest\x1a!.hubapi.RunCodeCollectorsResponse\"\x00\x12Q\n\x0e\x43urrentVersion\x12\x1d.hubapi.CurrentVersionRequest\x1a\x1e.hubapi.CurrentVersionResponse\"\x00\x12\x63\n\x14\x43heckComponentStatus\x12#.hubapi.CheckComponentStatusRequest\x1a$.hubapi.CheckComponentStatusResponse\"\x00\x12N\n\rCollectPolicy\x12\x1c.hubapi.CollectPolicyRequest\x1a\x1d.hubapi.CollectPolicyResponse\"\x00\x42\nZ\x08./hubapib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'hub_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\010./hubapi'
  _globals['_COLLECTION_DIMENSIONSENTRY']._options = None
  _globals['_COLLECTION_DIMENSIONSENTRY']._serialized_options = b'8\001'
  _globals['_COLLECTIONSOURCE']._serialized_start=1553
  _globals['_COLLECTIONSOURCE']._serialized_end=1656
  _globals['_CHECKCOMPONENTSTATUSTYPE']._serialized_start=1659
  _globals['_CHECKCOMPONENTSTATUSTYPE']._serialized_end=1803
  _globals['_CHECKSTATUS']._serialized_start=1806
  _globals['_CHECKSTATUS']._serialized_end=1941
  _globals['_OP']._serialized_start=1944
  _globals['_OP']._serialized_end=2140
  _globals['_RUNCODECOLLECTORSREQUEST']._serialized_start=54
  _globals['_RUNCODECOLLECTORSREQUEST']._serialized_end=127
  _globals['_CODECOLLECTIONRUN']._serialized_start=129
  _globals['_CODECOLLECTIONRUN']._serialized_end=206
  _globals['_RUNCODECOLLECTORSRESPONSE']._serialized_start=208
  _globals['_RUNCODECOLLECTORSRESPONSE']._serialized_end=312
  _globals['_COLLECTREQUEST']._serialized_start=314
  _globals['_COLLECTREQUEST']._serialized_end=384
  _globals['_COLLECTRESPONSE']._serialized_start=386
  _globals['_COLLECTRESPONSE']._serialized_end=403
  _globals['_PULLMANIFESTREQUEST']._serialized_start=405
  _globals['_PULLMANIFESTREQUEST']._serialized_end=455
  _globals['_PULLMANIFESTRESPONSE']._serialized_start=457
  _globals['_PULLMANIFESTRESPONSE']._serialized_end=503
  _globals['_COLLECTION']._serialized_start=506
  _globals['_COLLECTION']._serialized_end=920
  _globals['_COLLECTION_DIMENSIONSENTRY']._serialized_start=859
  _globals['_COLLECTION_DIMENSIONSENTRY']._serialized_end=920
  _globals['_CURRENTVERSIONREQUEST']._serialized_start=922
  _globals['_CURRENTVERSIONREQUEST']._serialized_end=945
  _globals['_CURRENTVERSIONRESPONSE']._serialized_start=947
  _globals['_CURRENTVERSIONRESPONSE']._serialized_end=1024
  _globals['_CHECKCOMPONENTSTATUSREQUEST']._serialized_start=1027
  _globals['_CHECKCOMPONENTSTATUSREQUEST']._serialized_end=1163
  _globals['_CHECKCOMPONENTSTATUSRESPONSE']._serialized_start=1165
  _globals['_CHECKCOMPONENTSTATUSRESPONSE']._serialized_end=1211
  _globals['_ASSERTIONRESULT']._serialized_start=1214
  _globals['_ASSERTIONRESULT']._serialized_end=1390
  _globals['_COLLECTPOLICYREQUEST']._serialized_start=1392
  _globals['_COLLECTPOLICYREQUEST']._serialized_end=1500
  _globals['_COLLECTPOLICYRESPONSE']._serialized_start=1502
  _globals['_COLLECTPOLICYRESPONSE']._serialized_end=1551
  _globals['_HUB']._serialized_start=2143
  _globals['_HUB']._serialized_end=2645
# @@protoc_insertion_point(module_scope)



import json
import semver

from src.lunar_policy import Check, Path, ComponentData


class TestCheckAssertions:
    def test_simple_value(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            c.assert_true(True)

        captured = capsys.readouterr()
        result = json.loads(captured.out)["assertions"][0]

        assert result["op"] == "true"
        assert result["args"] == ["True"]
        assert result["result"] == "pass"
        assert "failure_message" not in result

    def test_simple_path(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": True,
        }))

        with Check("test", data=data) as c:
            c.assert_true(Path(".value"))

        captured = capsys.readouterr()
        result = json.loads(captured.out)["assertions"][0]

        assert result["op"] == "true"
        assert result["args"] == ["True"]
        assert result["result"] == "pass"
        assert "failure_message" not in result

    def test_multiple_mixed_assertions_in_check(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": True,
        }))

        with Check("test", data=data) as c:
            c.assert_true(True)
            c.assert_true(Path(".value"))

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_true_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": True,
        }))

        with Check("test", data=data) as c:
            c.assert_true(True)
            c.assert_true(Path(".value"))

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_false_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": False,
        }))

        with Check("test", data=data) as c:
            c.assert_false(False)
            c.assert_false(Path(".value"))

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_equals_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": 1,
        }))

        with Check("test", data=data) as c:
            c.assert_equals(1, 1)
            c.assert_equals(Path(".value"), 1)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_greater_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": 1,
        }))

        with Check("test", data=data) as c:
            c.assert_greater(2, 1)
            c.assert_greater(Path(".value"), 0)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_greater_or_equal_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": 1,
        }))

        with Check("test", data=data) as c:
            c.assert_greater_or_equal(2, 1)
            c.assert_greater_or_equal(1, 1)
            c.assert_greater_or_equal(Path(".value"), 0)
            c.assert_greater_or_equal(Path(".value"), 1)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_less_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": 1,
        }))

        with Check("test", data=data) as c:
            c.assert_less(1, 2)
            c.assert_less(Path(".value"), 2)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_less_or_equal_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": 1,
        }))

        with Check("test", data=data) as c:
            c.assert_less_or_equal(1, 2)
            c.assert_less_or_equal(1, 1)
            c.assert_less_or_equal(Path(".value"), 2)
            c.assert_less_or_equal(Path(".value"), 1)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_contains_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": "hello",
        }))

        with Check("test", data=data) as c:
            c.assert_contains("hello", "e")
            c.assert_contains("hello", "hello")
            c.assert_contains("hello", "")
            c.assert_contains(Path(".value"), "e")

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_match_assertion(self, capsys):
        data = ComponentData.from_component_json(json.dumps({
            "value": "hello",
        }))

        with Check("test", data=data) as c:
            c.assert_match("hello", ".*ell.*")
            c.assert_match("hello", ".*hello.*")
            c.assert_match("hello", ".*")
            c.assert_match(Path(".value"), ".*ell.*")

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_foreign_type_comparison(self, capsys):
        data = ComponentData.from_component_json("{}")

        with Check("test", data=data) as c:
            v1 = semver.Version.parse("1.0.0")
            v2 = semver.Version.parse("2.0.0")

            c.assert_greater_or_equal(v2, v1)
            c.assert_less_or_equal(v1, v2)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)

    def test_all_instances_with_array_values(self, capsys):
        data = ComponentData.from_json(json.dumps({
            "merged_blob": {},
            "metadata_instances": [
                {
                    "payload": {
                        "value": [1, 2, 3]
                    }
                },
                {
                    "payload": {
                        "value": [1, 4, 5]
                    }
                }
            ]
        }))

        with Check("test", data=data) as c:
            c.assert_contains(Path(".value"), 1, all_instances=True)

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert all(result["result"] == "pass" for result in results)



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: hub.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

import google.protobuf.timestamp_pb2
import hub_pb2


class HubBase(abc.ABC):

    @abc.abstractmethod
    async def Collect(self, stream: 'grpclib.server.Stream[hub_pb2.CollectRequest, hub_pb2.CollectResponse]') -> None:
        pass

    @abc.abstractmethod
    async def PullManifest(self, stream: 'grpclib.server.Stream[hub_pb2.PullManifestRequest, hub_pb2.PullManifestResponse]') -> None:
        pass

    @abc.abstractmethod
    async def RunCodeCollectors(self, stream: 'grpclib.server.Stream[hub_pb2.RunCodeCollectorsRequest, hub_pb2.RunCodeCollectorsResponse]') -> None:
        pass

    @abc.abstractmethod
    async def CurrentVersion(self, stream: 'grpclib.server.Stream[hub_pb2.CurrentVersionRequest, hub_pb2.CurrentVersionResponse]') -> None:
        pass

    @abc.abstractmethod
    async def CheckComponentStatus(self, stream: 'grpclib.server.Stream[hub_pb2.CheckComponentStatusRequest, hub_pb2.CheckComponentStatusResponse]') -> None:
        pass

    @abc.abstractmethod
    async def CollectPolicy(self, stream: 'grpclib.server.Stream[hub_pb2.CollectPolicyRequest, hub_pb2.CollectPolicyResponse]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/hubapi.Hub/Collect': grpclib.const.Handler(
                self.Collect,
                grpclib.const.Cardinality.UNARY_UNARY,
                hub_pb2.CollectRequest,
                hub_pb2.CollectResponse,
            ),
            '/hubapi.Hub/PullManifest': grpclib.const.Handler(
                self.PullManifest,
                grpclib.const.Cardinality.UNARY_STREAM,
                hub_pb2.PullManifestRequest,
                hub_pb2.PullManifestResponse,
            ),
            '/hubapi.Hub/RunCodeCollectors': grpclib.const.Handler(
                self.RunCodeCollectors,
                grpclib.const.Cardinality.UNARY_UNARY,
                hub_pb2.RunCodeCollectorsRequest,
                hub_pb2.RunCodeCollectorsResponse,
            ),
            '/hubapi.Hub/CurrentVersion': grpclib.const.Handler(
                self.CurrentVersion,
                grpclib.const.Cardinality.UNARY_UNARY,
                hub_pb2.CurrentVersionRequest,
                hub_pb2.CurrentVersionResponse,
            ),
            '/hubapi.Hub/CheckComponentStatus': grpclib.const.Handler(
                self.CheckComponentStatus,
                grpclib.const.Cardinality.UNARY_UNARY,
                hub_pb2.CheckComponentStatusRequest,
                hub_pb2.CheckComponentStatusResponse,
            ),
            '/hubapi.Hub/CollectPolicy': grpclib.const.Handler(
                self.CollectPolicy,
                grpclib.const.Cardinality.UNARY_UNARY,
                hub_pb2.CollectPolicyRequest,
                hub_pb2.CollectPolicyResponse,
            ),
        }


class HubStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.Collect = grpclib.client.UnaryUnaryMethod(
            channel,
            '/hubapi.Hub/Collect',
            hub_pb2.CollectRequest,
            hub_pb2.CollectResponse,
        )
        self.PullManifest = grpclib.client.UnaryStreamMethod(
            channel,
            '/hubapi.Hub/PullManifest',
            hub_pb2.PullManifestRequest,
            hub_pb2.PullManifestResponse,
        )
        self.RunCodeCollectors = grpclib.client.UnaryUnaryMethod(
            channel,
            '/hubapi.Hub/RunCodeCollectors',
            hub_pb2.RunCodeCollectorsRequest,
            hub_pb2.RunCodeCollectorsResponse,
        )
        self.CurrentVersion = grpclib.client.UnaryUnaryMethod(
            channel,
            '/hubapi.Hub/CurrentVersion',
            hub_pb2.CurrentVersionRequest,
            hub_pb2.CurrentVersionResponse,
        )
        self.CheckComponentStatus = grpclib.client.UnaryUnaryMethod(
            channel,
            '/hubapi.Hub/CheckComponentStatus',
            hub_pb2.CheckComponentStatusRequest,
            hub_pb2.CheckComponentStatusResponse,
        )
        self.CollectPolicy = grpclib.client.UnaryUnaryMethod(
            channel,
            '/hubapi.Hub/CollectPolicy',
            hub_pb2.CollectPolicyRequest,
            hub_pb2.CollectPolicyResponse,
        )
